import React, { useState } from "react";
import { connect } from "react-redux";
import { Modal, Button, Form, Col} from "react-bootstrap";

import ErrorBoundary from "../../commons/components/error-boundary/ErrorBoundary";

import { CloseModal } from "../../state/modal/modal.action";

import { ModalType } from "../../enums/modal-type.enum";

import "./Modal.css";


function StrategyModal({props, dispatch}) {
    const [validated, setValidated] = useState(false);
    const { groups, name } = props;

    const onChange = (event) => {
        const form = event.currentTarget;
        const regex = /^[0-9a-zA-Z(\- _)]+$/;
        
        var input = undefined
        if (event.target.id === "strategyName") {
            input = event.target.value || "";
        }

        setValidated(form.checkValidity() && input.match(regex) && input != name);
    }

    const onSubmit = (event) => {
        event.preventDefault();
        event.stopPropagation();

        if (event.currentTarget.checkValidity() === true) {
            const data = new FormData(event.target);
            const entries = Object.fromEntries(data.entries());
            dispatch(props.onSubmit(entries));
            dispatch(CloseModal());
        }
    };

    return (
        <Form noValidate validated={validated} onSubmit={onSubmit} onChange={onChange}>
            <Form.Row>
                <Form.Group as={Col} controlId="strategyName">
                <Form.Control
                    required
                    maxLength="128"
                    minLength="4"
                    name="name"
                    type="text"
                    placeholder="Enter name"
                    defaultValue={name || ""}
                />
                <Form.Control.Feedback>Looks good!</Form.Control.Feedback>
                </Form.Group>
            </Form.Row>
            
            {
                groups &&
                <Form.Row>
                    <Form.Group as={Col} controlId="strategyGroup">
                        <Form.Control
                            as="select"
                            name="groupId"
                            required
                        >
                            {
                                groups.map((group, index) => (
                                    <option key={group.symbol} value={group.symbol}>{group.name}</option>
                                ))
                            }
                        </Form.Control>
                    </Form.Group>
                </Form.Row>
            }
            <Form.Row className="buttons-right">
                <Button variant="outline-secondary" onClick={() => dispatch(CloseModal(props.onClose))}>
                    Cancel
                </Button>
                <Button disabled={!validated} variant="outline-success" type="submit">
                    Save
                </Button>
            </Form.Row>
        </Form>
    );
}

function ConfirmModal({props, dispatch}) {

    return (
        <Form>
            <Form.Group>
                {props.message}
            </Form.Group>
            <Form.Row className="buttons-right">
                <Button variant="outline-secondary" onClick={() => dispatch(CloseModal(props.onClose))}>
                    Cancel
                </Button>
                <Button variant="outline-danger" onClick={() => {dispatch(props.onSubmit()); dispatch(CloseModal(props.onClose))} }>
                    Delete
                </Button>
            </Form.Row>
        </Form>
    );
}

function MessageModal({props, dispatch}) {
    return (
        <Form>
            <Form.Group>
                {props.message}
            </Form.Group>
            <Form.Row className="buttons-right">
                <Button variant="outline-primary" onClick={() => dispatch(CloseModal(props.onClose))}>
                    Close
                </Button>
            </Form.Row>
        </Form>
    );
}

const mapStateToProps = state => {
    return { modal: state.modal};
  };

const mapDispatchToProps = dispatch => {
    return {
        dispatch: (action) => {dispatch(action)},
    };
}

function Render({render, props, dispatch}) {
    switch(render) {
        case ModalType.Message:
            return <MessageModal props={props} dispatch={dispatch}/>
        case ModalType.Confirm:
            return <ConfirmModal props={props} dispatch={dispatch}/>
        case ModalType.Strategy:
            return <StrategyModal props={props} dispatch={dispatch}/>
        default:
            return <div></div>
    }
}

function ConnectedModal({modal, dispatch}) {
    return (
        <Modal
            show={modal.show } 
            onHide={() => dispatch(CloseModal(modal.onHide))}
            onShow={() => modal.onShow ? dispatch(modal.onShow()) : {}}
            size="md"
            aria-labelledby="contained-modal-title-vcenter"
            centered>
            <Modal.Header closeButton>
                <Modal.Title id="contained-modal-title-vcenter">{modal.title}</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <ErrorBoundary>
                    <Render render={modal.render} props={modal.props} dispatch={dispatch}/>
                </ErrorBoundary>
            </Modal.Body>
        </Modal>
    );
}

const GenericModal = connect(mapStateToProps, mapDispatchToProps)(ConnectedModal)

export default GenericModal;
