import React, { useEffect, useState } from "react";
import Loader from "react-loader-spinner";
import LoadingOverlay from "react-loading-overlay";
import { Card } from "react-bootstrap";
import { connect } from "react-redux";
import { CgClose } from "react-icons/cg";
import { VscScreenFull } from "react-icons/vsc";
import { BiRefresh } from "react-icons/bi";

import {getCountryMoneyFormat} from "../../../utils/currency.format";

import ErrorBoundary from "../../../commons/components/error-boundary/ErrorBoundary";
import BaseAccordion from "../../../commons/components/accordions/BaseAccordion";
import BaseTable from "../../../commons/components/table/BaseTable";
import ButtonIcon from "../../../commons/components/buttons/button-icon/ButtonIcon";
import BaseChart from "../../../commons/components/charts/BaseChart";

import { getMarketColumns } from "../../../enums/market-columns.enum";
import { getPositionColumns } from "../../../enums/position-columns.enum";
import { PositionOperation, PositionStatus } from "../../../enums/option-type.enum";

import { LoadStrategy, LoadStrategyChart, 
    ToggleStrategyCard, ToggleStrategyChart, 
    UpdateStrategy, UpdateStrategyCard } from "../../../state/strategy/strategy.action";
import { OpenMarketTab } from "../../../state/market/market.action";
import { NavigateTo } from "../../../state/app.action";


import "./StrategyCard.css";


function Controls({onRefresh, onResize, onClose}) {
    return (
        <div className="app-strategy-card--controls">
            <div className="app-strategy-card--buttons">
                <ButtonIcon 
                    theme="dark"
                    fontSize={20}
                    onClick={onRefresh}>
                        <BiRefresh/>
                </ButtonIcon>
                <ButtonIcon 
                    theme="dark"
                    check={true}
                    fontSize={20}
                    onClick={onResize}>
                        <VscScreenFull/>
                </ButtonIcon>
                <ButtonIcon 
                    theme="dark" 
                    fontSize={20} 
                    onClick={onClose}>
                        <CgClose/>
                </ButtonIcon>
            </div>
        </div>
    )
}

function Header({title, price, group, market, stats, positions}) {
    const {original} = stats
    return (
        <div className="app-strategy-card--header mb-3">
            <div className="app-strategy-card--info">
                <Card.Title>{title}</Card.Title>
                <Card.Text>{group ? group.name : "-"}</Card.Text>
                <Card.Text><b>{price}</b></Card.Text>
            </div>
            <div className="app-strategy-card--stats">
                <Card.Title className={`${original.possibleROI >= original.cost ? "green" : "red"}`}>{getCountryMoneyFormat(market.country, original.possibleROI - original.cost)}</Card.Title>
                <Card.Text>Costs: {getCountryMoneyFormat(market.country, original.cost - original.effectiveROI)}</Card.Text>
                <Card.Text>Tot. Positions: {positions.length}</Card.Text>
            </div>
        </div>      
    );
};

function ConnectedStrategyCard({card, dispatch}) {
    const {full, whatif, failed, loading, id, name, price, group, markets, positions, stats, chart} = card;
    const [orders, setOrders] = useState({});
    
    const changeOrders = (order) => {
        orders[order.id] = {...orders[order.id], ...order};
        setOrders(orders);
    };

    const submitOrders = (order=undefined) => {
        if (order !== undefined)
            orders[order.id] = {...orders[order.id], ...order};

        let submits = [];

        positions.forEach(position => {

            if (position.id in orders) {
                let order = orders[position.id];
                
                if (order.quantity != 0) {
                    submits.push(order);
    
                    if ( ((order.status != position.status || order.operation === PositionOperation.Delete)
                    && Math.abs(order.quantity) < Math.abs(position.quantity))) {
                        submits.push({...order, operation: PositionOperation.Add, status: position.status, quantity: position.quantity - order.quantity});
                    }
                }
            }
        });
        if (submits.length > 0) {
            dispatch(UpdateStrategy(id, submits, {...whatif, enabled: false}));
        }
        setOrders({})
    };

    // Refresh the card on first time rendering
    useEffect(() => dispatch(LoadStrategy(id)), [card.id]);

    return (
        <Card className={`app-strategy-card`}>
            <LoadingOverlay
                active={loading}
                spinner={<Loader className="app-accordion--loader" type="ThreeDots" color="#bbbbbb" height={100} width={100}/>}
                fadeSpeed={0}
            >
                <Controls
                    onRefresh={() => dispatch(LoadStrategy(id))}
                    onResize={() => {
                        dispatch(UpdateStrategyCard({...card, full: !full}));
                        dispatch(ToggleStrategyChart(id)); 
                    }}
                    onClose={() => dispatch(ToggleStrategyCard(card, true))}
                />
                <Card.Body>
                    <ErrorBoundary failed={failed}>
                        <Header
                            title={name}
                            price={price}
                            group={group}
                            stats={stats}
                            positions={positions}
                            market={markets && markets.length > 0 ? markets[0] : {}}
                        />
                        <BaseAccordion title="Chart" as={Card.Link} open={true}>
                            <BaseChart 
                                chart={chart}
                                onSelect={() => dispatch(LoadStrategyChart(id, chart.id))}
                            />
                        </BaseAccordion>
                        <BaseAccordion title="Markets" as={Card.Link}>
                            <BaseTable
                                columns={getMarketColumns(full, (market) => {
                                        dispatch(OpenMarketTab({...market, expiration: undefined, date: undefined, strategyId: id}));
                                        dispatch(NavigateTo("/app/markets"));
                                    }
                                )}
                                data={markets}
                                pagination={false}
                                >
                            </BaseTable>
                        </BaseAccordion>
                        <BaseAccordion title="Opens" as={Card.Link}>
                            <BaseTable
                                columns={getPositionColumns(full, changeOrders, submitOrders)}
                                data={positions.filter(p => p.status != PositionStatus.Close)}
                                pagination={false}
                                >
                            </BaseTable>
                        </BaseAccordion>
                    </ErrorBoundary>
                </Card.Body>
            </LoadingOverlay>
        </Card>
    );
};

const mapDispatchToProps = dispatch => {
    return {
        dispatch: (action) => {dispatch(action)},
    };
}

const StrategyCard = connect(null, mapDispatchToProps)(ConnectedStrategyCard)

export default StrategyCard;
