from bson.objectid import ObjectId
from formulas.base import get_security_underlying_price

from tasks import *
from formulas import get_implied_volatility, get_time_value

from chains.query import select_chain_option_by_contract
from markets.query import select_future_by_contract, select_market

from utils.mongodb import db
from utils.time import *

from .pipelines import *


# PORTFOLIO

def insert_portfolio(userId, name="My Portfolio", value=100000, currency="EUR"):
    return db.portfolio.insert_one(
    {
        'userId': userId,
        'name': name,
        'value': value,
        'currency': currency,
        'created': get_now(),
        'strategies': [],
    })


def select_portfolio(userId):
    portfolios = list(db.portfolio.aggregate([
        {"$match": {"userId": userId}},
        {"$lookup": {
            "from": "strategy",
            "let": {"strategies": "$strategies"},
            "pipeline": [
                {"$match": {"$expr": {"$in":["$_id", "$$strategies"]}}},
                {"$lookup": get_strategy_group_lookup()},
                {"$unwind": {
                    "path": "$positions",
                    "preserveNullAndEmptyArrays": True
                }},
                {"$lookup": get_strategy_market_lookup()},
                {"$lookup": get_strategy_chain_lookup()},
                {"$lookup": get_strategy_future_lookup()},
                {"$sort": {"status": -1, "endDate": -1, "startDate": -1}},
                {"$group": {
                    "_id": {
                        "id": "$_id",
                        "name": "$name",
                        "disabled": "$disabled",
                        "created": "$created",
                        "group": {"$arrayElemAt": ["$groups", 0]},
                    },
                    "positions": {"$push": {"$cond": [
                        {"$or": [{"$eq": ["$positions.status", TEMP]}, {"$eq": ["$positions.status", OPEN]}, {"$eq": ["$positions.status", CLOSE]}]},
                        {"$mergeObjects": [ "$positions", {"$arrayElemAt": ["$markets", 0]}, {"$arrayElemAt": ["$options",0]}, {"$arrayElemAt": ["$futures", 0]}]},
                        "$$REMOVE",
                    ]}}
                }},
                {"$project": {
                    "_id": "$_id.id",
                    "name": "$_id.name",
                    "disabled": "$_id.disabled",
                    "created": "$_id.created",
                    "group": "$_id.group",
                    "positions": 1,
                    "total": {"$cond": { "if": { "$isArray": "$positions" }, "then": { "$size": "$positions" }, "else": 0} },
                }},
                {"$sort": {"name": 1, "created": -1, "total": 1}}
            ],
            "as": "strategies",
        }}
    ]))
    return portfolios[0] if len(portfolios) == 1 else None

def save_portfolio_strategy(userId, strategyId):
    return db.portfolio.update_one({
        "userId": userId
    }, {
        "$addToSet": {"strategies": strategyId}
    })

def delete_portfolio_strategy(userId, strategyId):
    return db.portfolio.update_one({
        "userId": userId
    }, {
        "$pull": {"strategies": strategyId}
    })

# STRATEGIES

def select_strategies(userId, groupId, search):
    filter = {'userId': userId}
    
    if groupId is not None and groupId != '':
        filter["groupId"] = groupId
    
    if search is not None and search != '':
        filter['$text']= {'$search': search}

    return list(db.strategy.aggregate([
        {"$match": filter},
    ]))

# STRATEGY

def insert_strategy(data):
    return db.strategy.insert_one({
        'userId': data["userId"],
        'groupId': data["groupId"],
        'name': data["name"],
        'positions': [],
        'created': get_now(),
        'disabled': False,
        'closed': False,
        'whatif': {
            'enabled': False,
            'volatilityStep': 1,
            'volatilityIncrease': 0,
        }
    })

def remove_strategy(strategy):
    return db.strategy.delete_one({'_id': strategy["_id"], 'userId': strategy["userId"]})

def select_strategy(id, userId):
    strategies = list(db.strategy.aggregate([
        {"$match": {"_id": ObjectId(id), 'userId': userId}},
        {"$lookup": get_strategy_group_lookup()},
        {"$project": {
            "_id": 1,
            "groupId": 1,
            "userId": 1,
            "name": 1,
            "whatif": 1,
            "group": {"$arrayElemAt": ["$groups", 0]},
            "total": {"$cond": { "if": { "$isArray": "$positions" }, "then": { "$size": "$positions" }, "else": 0} },
        }}
    ]))
    return strategies[0] if len(strategies) > 0 else None

def select_strategy_positions(strategy, filter_active=None, filter_status=None):
    price = None
    whatif = strategy["whatif"]

    positions = list(db.strategy.aggregate(get_aggregate_strategy_positions(strategy)))
    
    for position in positions:
        # Update from whatif query params
        if whatif is not None:
            for key in whatif:
                position["whatif"][key] = whatif[key]

        if position["status"] != CLOSE:
            position["volatility"] = get_implied_volatility(position, price_key="startPrice")
            position["timeValue"] = get_time_value(position, price_key="startPrice")

        if price is None or ("template" in position and position["template"] == "index"):
            price = get_security_underlying_price(position)

    return price, list(filter(lambda p: 
                        (filter_active is None or p["active"] == filter_active) and
                        (filter_status is None or p["status"] != filter_status), positions))

## STRATEGY NAME

def save_strategy_name(strategy, name):
    return db.strategy.update_one(
    {
        '_id': strategy["_id"],
        'userId': strategy["userId"],
        'groupId': strategy["groupId"]
    }, {
        '$set': {'name': name}
    }, upsert=False)

## STRATEGY STATE

def save_strategy_state(strategy, disabled):
    return db.strategy.update_one(
    {
        '_id': strategy["_id"],
        'userId': strategy["userId"],
        'groupId': strategy["groupId"]
    }, {
        '$set': {'disabled': disabled}
    }, upsert=False)


## STRATEGY NAME

def save_strategy_whatif(strategy, whatif):
    return db.strategy.update_one(
    {
        '_id': strategy["_id"],
        'userId': strategy["userId"],
        'groupId': strategy["groupId"]
    }, {
        '$set': {'whatif': whatif}
    }, upsert=False)


## STRATEGY POSITION

def select_strategy_security_order(strategy, order):
    # From the contract of the order retrieve security product
    security = None

    if "contract" in order and order["contract"] is not None:
        security = select_future_by_contract(order["contract"])

        if security is None:
            security = select_chain_option_by_contract(order["contract"])
        
        if security is not None:
            market = select_market(security["symbol"])

            if market is None or market["groupId"] != strategy["groupId"]:
                security = None

    return security

def select_strategy_position(strategy, security, order):
    # Select the single position of a strategy (if exist)
    positions = list(db.strategy.aggregate([
        {"$match":{"_id": strategy["_id"], "userId": strategy["userId"]}},
        {"$unwind": "$positions"},
        {"$match": {"positions.id": order["id"]}},
        {"$replaceRoot": {"newRoot": "$positions"}}
    ]))

    if len(positions) == 1:
        position = positions[0]
    else:
        position = new_strategy_position(security, order)

    return set_strategy_position(position, order)

def save_strategy_position(strategy, position, order):
    # Save a single strategy position
    result = None

    if order["operation"] == ADD and position["quantity"] != 0:
        result = insert_strategy_position(strategy, position)

    if order["operation"] == UPDATE:
        result = update_srategy_position(strategy, position)

        if position["status"] == OPEN and position["active"] is True:
            result = open_strategy_position(strategy, position)

        if position["status"] == CLOSE and position["active"] is True:
            result = close_strategy_position(strategy, position)

    if order["operation"] == DELETE:
        result = delete_strategy_position(strategy, position)

    return result

def insert_strategy_position(strategy, position):
    # Add strategy position if not present
    return db.strategy.update_one(
    {
        "_id": strategy["_id"],
        "userId": strategy["userId"],
        "groupId": strategy["groupId"],
    }, {
        "$addToSet": {"positions": position}
    })

def update_srategy_position(strategy, position):
    # Update strategy position if present and temporary
    result = None
    if strategy["whatif"]["enabled"]:
        result = db.strategy.update_one({
            "_id": strategy["_id"],
            "userId": strategy["userId"],
            "groupId": strategy["groupId"],
            "positions": { "$elemMatch": {"id": position["id"]}}
        }, {
            "$set": {
                "positions.$.whatif": position["whatif"],
            }
        })
    else :
        if "startPrice" not in position:
            position["startPrice"] = position["price"]
            
        result = db.strategy.update_one({
            "_id": strategy["_id"],
            "userId": strategy["userId"],
            "groupId": strategy["groupId"],
            "positions": { "$elemMatch": {"id": position["id"]}}
        }, {
            "$set": {
                "positions.$.active": position["active"],
                "positions.$.quantity": position["quantity"],
                "positions.$.price": position["price"],
                "positions.$.startPrice": position["startPrice"],
            }
        })
    return result

def open_strategy_position(strategy, position):
    # Update strategy position if present and want to open (also in whatifs array)
    startDate = get_now()

    return db.strategy.update_one(
    {
        "_id": strategy["_id"],
        "userId": strategy["userId"],
        "groupId": strategy["groupId"],
        "positions": { "$elemMatch": {"id": position["id"], "status": TEMP}}
    }, {
        "$set": {
            "positions.$.status": OPEN,
            "positions.$.startDate": startDate,
            "positions.$.id": f'{position["contract"]}-{get_timestamp(startDate)}'
        }
    })

def close_strategy_position(strategy, position):
    # Update strategy position if present and want to close
    endDate = get_now()
    return db.strategy.update_one(
    {
        "_id": strategy["_id"],
        "userId": strategy["userId"],
        "groupId": strategy["groupId"],
        "positions": { "$elemMatch": {"id": position["id"], "status": OPEN}}
    }, {
        "$set": {
            "positions.$.status": CLOSE,
            "positions.$.endDate": endDate,
            "positions.$.endPrice": position["price"],
            "positions.$.id": f'{position["contract"]}-{get_timestamp(endDate)}'
        }
    })

def delete_strategy_position(strategy, position):
    # Remove strategy position if present and temporary
    return db.strategy.update_one(
    {
        "_id": strategy["_id"],
        "userId": strategy["userId"],
    }, {
        "$pull": {"positions": {"id": position["id"]}}
    })

def new_strategy_position(security, order):
    price = get_security_price(security)
    return {
        "active": True,
        "status": TEMP,      
        "id": order["id"],
        "contract": order["contract"],
        "quantity": order["quantity"],
        "exchange": security["exchange"],
        "symbol": security["symbol"],
        "expiration": security["expiration"],
        "date": security["date"],
        "type": security["type"],
        "strike": security["strike"],
        "price": price,
        "startPrice": price,
        "whatif": {
            "active": True,
            "quantity": order["quantity"],
            "price": price,
            "startPrice": price,
        }
    }

def set_strategy_position(position, order):
    
    if "active" in order and order["active"] is not None:
        position["active"] = order["active"]

    if "status" in order and order["status"] is not None:
        position["status"] = order["status"]

    if "quantity" in order and order["quantity"] is not None:
        position["quantity"] = order["quantity"]

    if "whatif" in order and order["whatif"] is not None:
        for key in order["whatif"]:
            if order["whatif"][key] is not None:
                position["whatif"][key] = order["whatif"][key]

    if "price" in order and order["price"] is not None:
        position["price"] = order["price"]

    if "startPrice" in order and order["startPrice"] is not None:
        position["startPrice"] = order["startPrice"]
    
    return position