from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

from tasks import *
from formulas import *

from chains.query import *
from markets.query import *

from .query import *
from .serializers import *

# USER
@api_view(['GET'])
def user(request):
    user = User.objects.get(username=request.user.username)
    serializer = GetUserSerializer(user)
    return Response(serializer.data)

# PORTOFLIO
@api_view(['GET'])
def portfolio(request):
    response = None
    userId = request.user.username

    portfolio = select_portfolio(userId)
    
    if portfolio is not None:            
        portfolio["stats"] = get_portfolio_stats(portfolio)
        serializer = GetPortfolioSerializer(portfolio)
        response = Response(serializer.data)

    else:
        response = Response({'message': 'Portfolio not found'},
                        status=status.HTTP_404_NOT_FOUND)

    return response

# STRATEGIES
def get_strategies(request):
    userId = request.user.username
    groupId = request.query_params.get('groupId')
    search = request.query_params.get('search')
    strategies = select_strategies(userId, groupId, search)
    serializer = GetStrategiesSerializer(strategies, many=True)
    return Response(serializer.data)


def post_strategies(request):
    userId = request.user.username

    if userId is not None:
        serializer = PostStrategiesSerializer(data=request.data)
        response = None

        if serializer.is_valid():
            data = serializer.validated_data
            data["userId"] = userId
            strategy = insert_strategy(data)

            if strategy.inserted_id is not None:
                save_portfolio_strategy(userId, strategy.inserted_id)

            response = Response({'id': str(strategy.inserted_id)}, status=status.HTTP_201_CREATED)
        
        else:
            response = Response({'message': "Request body is not valid", 'errors': serializer.error_messages},
                        status=status.HTTP_400_BAD_REQUEST)
    else:
        response = Response({'message': "User not found", 'errors': []},
                        status=status.HTTP_404_NOT_FOUND)
    return response


@api_view(['GET', 'POST'])
def strategies(request):
    response = None 

    if request.method == 'GET':
        response = get_strategies(request)

    elif request.method == 'POST':
        response = post_strategies(request)

    return response

# STRATEGY

def get_strategy(strategy):
    if strategy is not None:
        # Get context info
        strategy["markets"] = select_markets(strategy["groupId"])

        # Get complete info from chain or future for each position
        price, positions = select_strategy_positions(strategy)     
        strategy["positions"] = positions
        strategy["price"] = price

        # Get statistical info about the strategy
        strategy["stats"] = get_strategy_stats(strategy)

    serializer = GetStrategySerializer(strategy, allow_null=True)
    return Response(serializer.data)

def post_strategy(strategy, data):
    message = "Nothing to change"
    code = status.HTTP_200_OK
    errors = []

    serializer = PostStrategySerializer(data=data, allow_null=True)

    if serializer.is_valid():
        data = serializer.validated_data
        securities = []
        orders = []

        if "orders" in data and data["orders"] is not None:
            orders = data["orders"]

            for order in orders:
                security = select_strategy_security_order(strategy, order)

                if security is not None:
                    securities.append(security)
                
        if len(securities) == len(orders):            
            result = None

            for security, order in zip(securities, orders):
                position = select_strategy_position(strategy, security, order)
                result = save_strategy_position(strategy, position, order)

            if "name" in data and data["name"] is not None:
                result = save_strategy_name(strategy, data["name"])

            if "disabled" in data and data["disabled"] is not None:
                result = save_strategy_state(strategy, data["disabled"])

            if "whatif" in data and data["whatif"] is not None:
                strategy["whatif"] = data["whatif"]
                result = save_strategy_whatif(strategy, data["whatif"])

            if result is not None:
                message = f"Modified {result.modified_count} document/s and saved {len(securities)} orders"
        else:
            message = f"There are some invalid order contracts"
            code = status.HTTP_404_NOT_FOUND

    else:
        message = "Request body is not valid"
        errors = serializer.errors 
        code = status.HTTP_400_BAD_REQUEST
        
    return Response({'message': message, 'errors': errors}, status=code)

def delete_strategy(strategy):
    result = remove_strategy(strategy)
    count = 0
    if result:
        count = result.deleted_count
        if count == 1:
            delete_portfolio_strategy(strategy["userId"], strategy["_id"])
            
    return Response({'message': f'Deleted {count} strategies'}, status=status.HTTP_200_OK)


@api_view(['GET', 'POST', 'DELETE'])
def strategy(request, id):
    response = None 
    userId = request.user.username

    strategy = select_strategy(id, userId)

    if strategy is not None:

        if request.method == 'GET':
            response = get_strategy(strategy)

        elif request.method == 'POST':
            data = request.data
            response = post_strategy(strategy, data)
        
        elif request.method == 'DELETE':
            response = delete_strategy(strategy)

    else:
        response = Response({'message': 'Strategy not found'},
                        status=status.HTTP_404_NOT_FOUND)
   
    return response

# PROFIT

@api_view(['GET'])
def strategy_profit(request, id):
    response = None 
    userId = request.user.username

    strategy = select_strategy(id, userId)

    if strategy is not None:
        # Select positions with aggregated informations
        price, positions = select_strategy_positions(strategy)

        # Get profits
        profits = get_profits(positions)

        # Search the price index inside the profits
        index = binary_search(profits, price, "price")
    
        serializer = GetStrategyProfitSerializer({"price": price, "index": index, "profits": profits, "isWhatif": strategy["whatif"]["enabled"]}, allow_null=True)
        response = Response(serializer.data, status=status.HTTP_200_OK)

    else:
        response = Response({'message': 'Strategy not found'},
                        status=status.HTTP_404_NOT_FOUND)
   
    return response

# GREEKS

@api_view(['GET'])
def strategy_greeks(request, id):
    response = None 
    userId = request.user.username

    strategy = select_strategy(id, userId)

    if strategy is not None:
        # Get positions and underlying price
        price, positions = select_strategy_positions(strategy)

        # Get greeks
        greeks = get_greeks(positions)

        # Search the price index inside the greeks
        index = binary_search(greeks, price, "price")

        serializer = GetStrategyGreeksSerializer({"index": index, "price": price, "greeks": greeks}, allow_null=True)
        response = Response(serializer.data, status=status.HTTP_200_OK)

    else:
        response = Response({'message': 'Strategy not found'},
                        status=status.HTTP_404_NOT_FOUND)
   
    return response
